@@ -1,5 +1,3 @@
-/* $Id: network_chat_gui.cpp 23740 2012-01-03 21:32:51Z rubidium $ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -9,10 +7,6 @@
 
 /** @file network_chat_gui.cpp GUI for handling chat messages. */
 
-#include <stdarg.h> /* va_list */
-
-#ifdef ENABLE_NETWORK
-
 #include "../stdafx.h"
 #include "../strings_func.h"
 #include "../blitter/factory.hpp"
@@ -21,7 +15,11 @@
 #include "../querystring_gui.h"
 #include "../town.h"
 #include "../window_func.h"
+#include "../toolbar_gui.h"
 #include "../core/geometry_func.hpp"
+#include "../zoom_func.h"
+#include "../timer/timer.h"
+#include "../timer/timer_window.h"
 #include "network.h"
 #include "network_client.h"
 #include "network_base.h"
@@ -30,101 +28,81 @@
 
 #include "table/strings.h"
 
-/** The draw buffer must be able to contain the chat message, client name and the "[All]" message,
- * some spaces and possible translations of [All] to other languages. */
-assert_compile((int)DRAW_STRING_BUFFER >= (int)NETWORK_CHAT_LENGTH + NETWORK_NAME_LENGTH + 40);
+#include "../safeguards.h"
 
 /** Spacing between chat lines. */
 static const uint NETWORK_CHAT_LINE_SPACING = 3;
 
 /** Container for a message. */
 struct ChatMessage {
-	char message[DRAW_STRING_BUFFER]; ///< The action message.
+	std::string message; ///< The action message.
 	TextColour colour;  ///< The colour of the message.
-	uint32 remove_time; ///< The time to remove the message.
+	std::chrono::steady_clock::time_point remove_time; ///< The time to remove the message.
 };
 
 /* used for chat window */
-static ChatMessage *_chatmsg_list = NULL; ///< The actual chat message list.
+static std::deque<ChatMessage> _chatmsg_list; ///< The actual chat message list.
 static bool _chatmessage_dirty = false;   ///< Does the chat message need repainting?
 static bool _chatmessage_visible = false; ///< Is a chat message visible.
 static bool _chat_tab_completion_active;  ///< Whether tab completion is active.
 static uint MAX_CHAT_MESSAGES = 0;        ///< The limit of chat messages to show.
 
+/**
+ * Time the chat history was marked dirty. This is used to determine if expired
+ * messages have recently expired and should cause a redraw to hide them.
+ */
+static std::chrono::steady_clock::time_point _chatmessage_dirty_time;
+
 /**
  * The chatbox grows from the bottom so the coordinates are pixels from
  * the left and pixels from the bottom. The height is the maximum height.
  */
 static PointDimension _chatmsg_box;
-static uint8 *_chatmessage_backup = NULL; ///< Backup in case text is moved.
+static ReusableBuffer<uint8_t> _chatmessage_backup; ///< Backup in case text is moved.
 
 /**
- * Count the chat messages.
- * @return The number of chat messages.
+ * Test if there are any chat messages to display.
+ * @param show_all Set if all messages should be included, instead of unexpired only.
+ * @return True iff there are chat messages to display.
  */
-static inline uint GetChatMessageCount()
+static inline bool HaveChatMessages(bool show_all)
 {
-	uint i = 0;
-	for (; i < MAX_CHAT_MESSAGES; i++) {
-		if (_chatmsg_list[i].message[0] == '\0') break;
+	if (show_all) return !_chatmsg_list.empty();
+
+	auto now = std::chrono::steady_clock::now();
+	for (auto &cmsg : _chatmsg_list) {
+		if (cmsg.remove_time >= now) return true;
 	}
 
-	return i;
+	return false;
 }
 
 /**
  * Add a text message to the 'chat window' to be shown
  * @param colour The colour this message is to be shown in
  * @param duration The duration of the chat message in seconds
- * @param message message itself in printf() style
+ * @param message message itself
  */
-void CDECL NetworkAddChatMessage(TextColour colour, uint duration, const char *message, ...)
+void CDECL NetworkAddChatMessage(TextColour colour, uint duration, const std::string &message)
 {
-	char buf[DRAW_STRING_BUFFER];
-	const char *bufp;
-	va_list va;
-	uint msg_count;
-	uint16 lines;
-
-	va_start(va, message);
-	vsnprintf(buf, lengthof(buf), message, va);
-	va_end(va);
-
-	Utf8TrimString(buf, DRAW_STRING_BUFFER);
-
-	/* Force linebreaks for strings that are too long */
-	lines = GB(FormatStringLinebreaks(buf, lastof(buf), _chatmsg_box.width - 8), 0, 16) + 1;
-	if (lines >= MAX_CHAT_MESSAGES) return;
-
-	msg_count = GetChatMessageCount();
-	/* We want to add more chat messages than there is free space for, remove 'old' */
-	if (lines > MAX_CHAT_MESSAGES - msg_count) {
-		int i = lines - (MAX_CHAT_MESSAGES - msg_count);
-		memmove(&_chatmsg_list[0], &_chatmsg_list[i], sizeof(_chatmsg_list[0]) * (msg_count - i));
-		msg_count = MAX_CHAT_MESSAGES - lines;
+	if (_chatmsg_list.size() == MAX_CHAT_MESSAGES) {
+		_chatmsg_list.pop_back();
 	}
 
-	for (bufp = buf; lines != 0; lines--) {
-		ChatMessage *cmsg = &_chatmsg_list[msg_count++];
-		strecpy(cmsg->message, bufp, lastof(cmsg->message));
-
-		/* The default colour for a message is company colour. Replace this with
-		 * white for any additional lines */
-		cmsg->colour = (bufp == buf && (colour & TC_IS_PALETTE_COLOUR)) ? colour : TC_WHITE;
-		cmsg->remove_time = _realtime_tick + duration * 1000;
-
-		bufp += strlen(bufp) + 1; // jump to 'next line' in the formatted string
-	}
+	ChatMessage *cmsg = &_chatmsg_list.emplace_front();
+	cmsg->message = message;
+	cmsg->colour = colour;
+	cmsg->remove_time = std::chrono::steady_clock::now() + std::chrono::seconds(duration);
 
+	_chatmessage_dirty_time = std::chrono::steady_clock::now();
 	_chatmessage_dirty = true;
 }
 
 /** Initialize all font-dependent chat box sizes. */
 void NetworkReInitChatBoxSize()
 {
-	_chatmsg_box.y       = 3 * FONT_HEIGHT_NORMAL;
-	_chatmsg_box.height  = MAX_CHAT_MESSAGES * (FONT_HEIGHT_NORMAL + NETWORK_CHAT_LINE_SPACING) + 2;
-	_chatmessage_backup  = ReallocT(_chatmessage_backup, _chatmsg_box.width * _chatmsg_box.height * BlitterFactoryBase::GetCurrentBlitter()->GetBytesPerPixel());
+	_chatmsg_box.y       = 3 * GetCharacterHeight(FS_NORMAL);
+	_chatmsg_box.height  = MAX_CHAT_MESSAGES * (GetCharacterHeight(FS_NORMAL) + ScaleGUITrad(NETWORK_CHAT_LINE_SPACING)) + ScaleGUITrad(4);
 }
 
 /** Initialize all buffers of the chat visualisation. */
@@ -132,15 +110,11 @@ void NetworkInitChatMessage()
 {
 	MAX_CHAT_MESSAGES    = _settings_client.gui.network_chat_box_height;
 
-	_chatmsg_list        = ReallocT(_chatmsg_list, _settings_client.gui.network_chat_box_height);
-	_chatmsg_box.x       = 10;
-	_chatmsg_box.width   = _settings_client.gui.network_chat_box_width;
+	_chatmsg_list.clear();
+	_chatmsg_box.x       = ScaleGUITrad(10);
+	_chatmsg_box.width   = _settings_client.gui.network_chat_box_width_pct * _screen.width / 100;
 	NetworkReInitChatBoxSize();
 	_chatmessage_visible = false;
-
-	for (uint i = 0; i < MAX_CHAT_MESSAGES; i++) {
-		_chatmsg_list[i].message[0] = '\0';
-	}
 }
 
 /** Hide the chatbox */
@@ -149,7 +123,7 @@ void NetworkUndrawChatMessage()
 	/* Sometimes we also need to hide the cursor
 	 *   This is because both textmessage and the cursor take a shot of the
 	 *   screen before drawing.
-	 *   Now the textmessage takes his shot and paints his data before the cursor
+	 *   Now the textmessage takes its shot and paints its data before the cursor
 	 *   does, so in the shot of the cursor is the screen-data of the textmessage
 	 *   included when the cursor hangs somewhere over the textmessage. To
 	 *   avoid wrong repaints, we undraw the cursor in that case, and everything
@@ -165,13 +139,13 @@ void NetworkUndrawChatMessage()
 	}
 
 	if (_chatmessage_visible) {
-		Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
+		Blitter *blitter = BlitterFactory::GetCurrentBlitter();
 		int x      = _chatmsg_box.x;
 		int y      = _screen.height - _chatmsg_box.y - _chatmsg_box.height;
 		int width  = _chatmsg_box.width;
 		int height = _chatmsg_box.height;
 		if (y < 0) {
-			height = max(height + y, min(_chatmsg_box.height, _screen.height));
+			height = std::max(height + y, std::min(_chatmsg_box.height, _screen.height));
 			y = 0;
 		}
 		if (x + width >= _screen.width) {
@@ -181,57 +155,51 @@ void NetworkUndrawChatMessage()
 
 		_chatmessage_visible = false;
 		/* Put our 'shot' back to the screen */
-		blitter->CopyFromBuffer(blitter->MoveTo(_screen.dst_ptr, x, y), _chatmessage_backup, width, height);
+		blitter->CopyFromBuffer(blitter->MoveTo(_screen.dst_ptr, x, y), _chatmessage_backup.GetBuffer(), width, height);
 		/* And make sure it is updated next time */
-		_video_driver->MakeDirty(x, y, width, height);
+		VideoDriver::GetInstance()->MakeDirty(x, y, width, height);
 
+		_chatmessage_dirty_time = std::chrono::steady_clock::now();
 		_chatmessage_dirty = true;
 	}
 }
 
-/** Check if a message is expired. */
-void NetworkChatMessageLoop()
-{
-	for (uint i = 0; i < MAX_CHAT_MESSAGES; i++) {
-		ChatMessage *cmsg = &_chatmsg_list[i];
-		if (cmsg->message[0] == '\0') continue;
-
+/** Check if a message is expired on a regular interval. */
+static IntervalTimer<TimerWindow> network_message_expired_interval(std::chrono::seconds(1), [](auto) {
+	auto now = std::chrono::steady_clock::now();
+	for (auto &cmsg : _chatmsg_list) {
 		/* Message has expired, remove from the list */
-		if (cmsg->remove_time < _realtime_tick) {
-			/* Move the remaining messages over the current message */
-			if (i != MAX_CHAT_MESSAGES - 1) memmove(cmsg, cmsg + 1, sizeof(*cmsg) * (MAX_CHAT_MESSAGES - i - 1));
-
-			/* Mark the last item as empty */
-			_chatmsg_list[MAX_CHAT_MESSAGES - 1].message[0] = '\0';
+		if (now > cmsg.remove_time && _chatmessage_dirty_time < cmsg.remove_time) {
+			_chatmessage_dirty_time = now;
 			_chatmessage_dirty = true;
-
-			/* Go one item back, because we moved the array 1 to the left */
-			i--;
+			break;
 		}
 	}
-}
+});
 
 /** Draw the chat message-box */
 void NetworkDrawChatMessage()
 {
-	Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
+	Blitter *blitter = BlitterFactory::GetCurrentBlitter();
 	if (!_chatmessage_dirty) return;
 
+	const Window *w = FindWindowByClass(WC_SEND_NETWORK_MSG);
+	bool show_all = (w != nullptr);
+
 	/* First undraw if needed */
 	NetworkUndrawChatMessage();
 
 	if (_iconsole_mode == ICONSOLE_FULL) return;
 
 	/* Check if we have anything to draw at all */
-	uint count = GetChatMessageCount();
-	if (count == 0) return;
+	if (!HaveChatMessages(show_all)) return;
 
 	int x      = _chatmsg_box.x;
 	int y      = _screen.height - _chatmsg_box.y - _chatmsg_box.height;
 	int width  = _chatmsg_box.width;
 	int height = _chatmsg_box.height;
 	if (y < 0) {
-		height = max(height + y, min(_chatmsg_box.height, _screen.height));
+		height = std::max(height + y, std::min(_chatmsg_box.height, _screen.height));
 		y = 0;
 	}
 	if (x + width >= _screen.width) {
@@ -239,29 +207,40 @@ void NetworkDrawChatMessage()
 	}
 	if (width <= 0 || height <= 0) return;
 
-	assert(blitter->BufferSize(width, height) <= (int)(_chatmsg_box.width * _chatmsg_box.height * blitter->GetBytesPerPixel()));
-
 	/* Make a copy of the screen as it is before painting (for undraw) */
-	blitter->CopyToBuffer(blitter->MoveTo(_screen.dst_ptr, x, y), _chatmessage_backup, width, height);
+	uint8_t *buffer = _chatmessage_backup.Allocate(BlitterFactory::GetCurrentBlitter()->BufferSize(width, height));
+	blitter->CopyToBuffer(blitter->MoveTo(_screen.dst_ptr, x, y), buffer, width, height);
 
 	_cur_dpi = &_screen; // switch to _screen painting
 
+	auto now = std::chrono::steady_clock::now();
+	int string_height = 0;
+	for (auto &cmsg : _chatmsg_list) {
+		if (!show_all && cmsg.remove_time < now) continue;
+		SetDParamStr(0, cmsg.message);
+		string_height += GetStringLineCount(STR_JUST_RAW_STRING, width - 1) * GetCharacterHeight(FS_NORMAL) + NETWORK_CHAT_LINE_SPACING;
+	}
+
+	string_height = std::min<uint>(string_height, MAX_CHAT_MESSAGES * (GetCharacterHeight(FS_NORMAL) + NETWORK_CHAT_LINE_SPACING));
+
+	int top = _screen.height - _chatmsg_box.y - string_height - 2;
+	int bottom = _screen.height - _chatmsg_box.y - 2;
 	/* Paint a half-transparent box behind the chat messages */
-	GfxFillRect(
-			_chatmsg_box.x,
-			_screen.height - _chatmsg_box.y - count * (FONT_HEIGHT_NORMAL + NETWORK_CHAT_LINE_SPACING) - 2,
-			_chatmsg_box.x + _chatmsg_box.width - 1,
-			_screen.height - _chatmsg_box.y - 2,
+	GfxFillRect(_chatmsg_box.x, top - 2, _chatmsg_box.x + _chatmsg_box.width - 1, bottom,
 			PALETTE_TO_TRANSPARENT, FILLRECT_RECOLOUR // black, but with some alpha for background
 		);
 
 	/* Paint the chat messages starting with the lowest at the bottom */
-	for (uint y = FONT_HEIGHT_NORMAL + NETWORK_CHAT_LINE_SPACING; count-- != 0; y += (FONT_HEIGHT_NORMAL + NETWORK_CHAT_LINE_SPACING)) {
-		DrawString(_chatmsg_box.x + 3, _chatmsg_box.x + _chatmsg_box.width - 1, _screen.height - _chatmsg_box.y - y + 1, _chatmsg_list[count].message, _chatmsg_list[count].colour);
+	int ypos = bottom - 2;
+
+	for (auto &cmsg : _chatmsg_list) {
+		if (!show_all && cmsg.remove_time < now) continue;
+		ypos = DrawStringMultiLine(_chatmsg_box.x + ScaleGUITrad(3), _chatmsg_box.x + _chatmsg_box.width - 1, top, ypos, cmsg.message, cmsg.colour, SA_LEFT | SA_BOTTOM | SA_FORCE) - NETWORK_CHAT_LINE_SPACING;
+		if (ypos < top) break;
 	}
 
 	/* Make sure the data is updated next flush */
-	_video_driver->MakeDirty(x, y, width, height);
+	VideoDriver::GetInstance()->MakeDirty(x, y, width, height);
 
 	_chatmessage_visible = true;
 	_chatmessage_dirty = false;
@@ -273,9 +252,9 @@ void NetworkDrawChatMessage()
  * @param type The type of destination.
  * @param dest The actual destination index.
  */
-static void SendChat(const char *buf, DestType type, int dest)
+static void SendChat(const std::string &buf, DestType type, int dest)
 {
-	if (StrEmpty(buf)) return;
+	if (buf.empty()) return;
 	if (!_network_server) {
 		MyClient::SendChat((NetworkAction)(NETWORK_ACTION_CHAT + type), type, dest, buf, 0);
 	} else {
@@ -284,10 +263,10 @@ static void SendChat(const char *buf, DestType type, int dest)
 }
 
 /** Window to enter the chat message in. */
-struct NetworkChatWindow : public QueryStringBaseWindow {
+struct NetworkChatWindow : public Window {
 	DestType dtype;       ///< The type of destination.
-	StringID dest_string; ///< String representation of the destination.
 	int dest;             ///< The identifier of the destination.
+	QueryString message_editbox; ///< Message editbox.
 
 	/**
 	 * Create a chat input window.
@@ -295,12 +274,13 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 	 * @param type The type of destination.
 	 * @param dest The actual destination index.
 	 */
-	NetworkChatWindow(const WindowDesc *desc, DestType type, int dest) : QueryStringBaseWindow(NETWORK_CHAT_LENGTH)
+	NetworkChatWindow(WindowDesc *desc, DestType type, int dest) : Window(desc), message_editbox(NETWORK_CHAT_LENGTH)
 	{
 		this->dtype   = type;
 		this->dest    = dest;
-		this->afilter = CS_ALPHANUMERAL;
-		InitializeTextBuffer(&this->text, this->edit_str_buf, this->edit_str_size);
+		this->querystrings[WID_NC_TEXTBOX] = &this->message_editbox;
+		this->message_editbox.cancel_button = WID_NC_CLOSE;
+		this->message_editbox.ok_button = WID_NC_SENDBUTTON;
 
 		static const StringID chat_captions[] = {
 			STR_NETWORK_CHAT_ALL_CAPTION,
@@ -308,9 +288,10 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 			STR_NETWORK_CHAT_CLIENT_CAPTION
 		};
 		assert((uint)this->dtype < lengthof(chat_captions));
-		this->dest_string = chat_captions[this->dtype];
 
-		this->InitNested(desc, type);
+		this->CreateNestedTree();
+		this->GetWidget<NWidgetCore>(WID_NC_DESTINATION)->widget_data = chat_captions[this->dtype];
+		this->FinishInitNested(type);
 
 		this->SetFocusedWidget(WID_NC_TEXTBOX);
 		InvalidateWindowData(WC_NEWS_WINDOW, 0, this->height);
@@ -319,26 +300,29 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 		PositionNetworkChatWindow(this);
 	}
 
-	~NetworkChatWindow()
+	void Close([[maybe_unused]] int data = 0) override
 	{
 		InvalidateWindowData(WC_NEWS_WINDOW, 0, 0);
+		this->Window::Close();
+	}
+
+	void FindWindowPlacementAndResize([[maybe_unused]] int def_width, [[maybe_unused]] int def_height) override
+	{
+		Window::FindWindowPlacementAndResize(_toolbar_width, def_height);
 	}
 
 	/**
 	 * Find the next item of the list of things that can be auto-completed.
 	 * @param item The current indexed item to return. This function can, and most
 	 *     likely will, alter item, to skip empty items in the arrays.
-	 * @return Returns the char that matched to the index.
+	 * @return Returns the view that matched to the index.
 	 */
-	const char *ChatTabCompletionNextItem(uint *item)
+	std::optional<std::string> ChatTabCompletionNextItem(uint *item)
 	{
-		static char chat_tab_temp_buffer[64];
-
 		/* First, try clients */
 		if (*item < MAX_CLIENT_SLOTS) {
 			/* Skip inactive clients */
-			NetworkClientInfo *ci;
-			FOR_ALL_CLIENT_INFOS_FROM(ci, *item) {
+			for (NetworkClientInfo *ci : NetworkClientInfo::Iterate(*item)) {
 				*item = ci->index;
 				return ci->client_name;
 			}
@@ -349,17 +333,14 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 		 * Not that the following assumes all town indices are adjacent, ie no
 		 * towns have been deleted. */
 		if (*item < (uint)MAX_CLIENT_SLOTS + Town::GetPoolSize()) {
-			const Town *t;
-
-			FOR_ALL_TOWNS_FROM(t, *item - MAX_CLIENT_SLOTS) {
+			for (const Town *t : Town::Iterate(*item - MAX_CLIENT_SLOTS)) {
 				/* Get the town-name via the string-system */
 				SetDParam(0, t->index);
-				GetString(chat_tab_temp_buffer, STR_TOWN_NAME, lastof(chat_tab_temp_buffer));
-				return &chat_tab_temp_buffer[0];
+				return GetString(STR_TOWN_NAME);
 			}
 		}
 
-		return NULL;
+		return std::nullopt;
 	}
 
 	/**
@@ -367,13 +348,14 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 	 *  the word right from that as to complete. It also writes a \0 at the
 	 *  position of the space (if any). If nothing found, buf is returned.
 	 */
-	static char *ChatTabCompletionFindText(char *buf)
+	static std::string_view ChatTabCompletionFindText(std::string_view &buf)
 	{
-		char *p = strrchr(buf, ' ');
-		if (p == NULL) return buf;
+		auto it = buf.find_last_of(' ');
+		if (it == std::string_view::npos) return buf;
 
-		*p = '\0';
-		return p + 1;
+		std::string_view res = buf.substr(it + 1);
+		buf.remove_suffix(res.size() + 1);
+		return res;
 	}
 
 	/**
@@ -381,46 +363,44 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 	 */
 	void ChatTabCompletion()
 	{
-		static char _chat_tab_completion_buf[NETWORK_CHAT_LENGTH];
-		assert(this->edit_str_size == lengthof(_chat_tab_completion_buf));
+		static std::string _chat_tab_completion_buf;
 
-		Textbuf *tb = &this->text;
-		size_t len, tb_len;
-		uint item;
-		char *tb_buf, *pre_buf;
-		const char *cur_name;
+		Textbuf *tb = &this->message_editbox.text;
+		uint item = 0;
 		bool second_scan = false;
 
-		item = 0;
+		/* Create views, so we do not need to copy the data for now. */
+		std::string_view pre_buf = _chat_tab_completion_active ? std::string_view(_chat_tab_completion_buf) : std::string_view(tb->buf);
+		std::string_view tb_buf = ChatTabCompletionFindText(pre_buf);
 
-		/* Copy the buffer so we can modify it without damaging the real data */
-		pre_buf = (_chat_tab_completion_active) ? strdup(_chat_tab_completion_buf) : strdup(tb->buf);
+		/*
+		 * Comparing pointers of the data, as both "Hi:<tab>" and "Hi: Hi:<tab>" will result in
+		 * tb_buf and pre_buf being "Hi:", which would be equal in content but not in context.
+		 */
+		bool begin_of_line = tb_buf.data() == pre_buf.data();
 
-		tb_buf  = ChatTabCompletionFindText(pre_buf);
-		tb_len  = strlen(tb_buf);
-
-		while ((cur_name = ChatTabCompletionNextItem(&item)) != NULL) {
+		std::optional<std::string> cur_item;
+		while ((cur_item = ChatTabCompletionNextItem(&item)).has_value()) {
+			std::string_view cur_name = cur_item.value();
 			item++;
 
 			if (_chat_tab_completion_active) {
 				/* We are pressing TAB again on the same name, is there another name
 				 *  that starts with this? */
 				if (!second_scan) {
-					size_t offset;
-					size_t length;
+					std::string_view view;
 
 					/* If we are completing at the begin of the line, skip the ': ' we added */
-					if (tb_buf == pre_buf) {
-						offset = 0;
-						length = (tb->bytes - 1) - 2;
+					if (begin_of_line) {
+						view = std::string_view(tb->buf, (tb->bytes - 1) - 2);
 					} else {
 						/* Else, find the place we are completing at */
-						offset = strlen(pre_buf) + 1;
-						length = (tb->bytes - 1) - offset;
+						size_t offset = pre_buf.size() + 1;
+						view = std::string_view(tb->buf + offset, (tb->bytes - 1) - offset);
 					}
 
 					/* Compare if we have a match */
-					if (strlen(cur_name) == length && strncmp(cur_name, tb->buf + offset, length) == 0) second_scan = true;
+					if (cur_name == view) second_scan = true;
 
 					continue;
 				}
@@ -428,119 +408,75 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 				/* Now any match we make on _chat_tab_completion_buf after this, is perfect */
 			}
 
-			len = strlen(cur_name);
-			if (tb_len < len && strncasecmp(cur_name, tb_buf, tb_len) == 0) {
+			if (tb_buf.size() < cur_name.size() && cur_name.starts_with(tb_buf)) {
 				/* Save the data it was before completion */
-				if (!second_scan) snprintf(_chat_tab_completion_buf, lengthof(_chat_tab_completion_buf), "%s", tb->buf);
+				if (!second_scan) _chat_tab_completion_buf = tb->buf;
 				_chat_tab_completion_active = true;
 
 				/* Change to the found name. Add ': ' if we are at the start of the line (pretty) */
-				if (pre_buf == tb_buf) {
-					snprintf(tb->buf, this->edit_str_size, "%s: ", cur_name);
+				if (begin_of_line) {
+					this->message_editbox.text.Assign(fmt::format("{}: ", cur_name));
 				} else {
-					snprintf(tb->buf, this->edit_str_size, "%s %s", pre_buf, cur_name);
+					this->message_editbox.text.Assign(fmt::format("{} {}", pre_buf, cur_name));
 				}
 
-				/* Update the textbuffer */
-				UpdateTextBufferSize(&this->text);
-
 				this->SetDirty();
-				free(pre_buf);
 				return;
 			}
 		}
 
 		if (second_scan) {
-			/* We walked all posibilities, and the user presses tab again.. revert to original text */
-			strcpy(tb->buf, _chat_tab_completion_buf);
+			/* We walked all possibilities, and the user presses tab again.. revert to original text */
+			this->message_editbox.text.Assign(_chat_tab_completion_buf);
 			_chat_tab_completion_active = false;
 
-			/* Update the textbuffer */
-			UpdateTextBufferSize(&this->text);
-
 			this->SetDirty();
 		}
-		free(pre_buf);
 	}
 
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
-		this->DrawEditBox(WID_NC_TEXTBOX);
-	}
-
-	virtual Point OnInitialPosition(const WindowDesc *desc, int16 sm_width, int16 sm_height, int window_number)
+	Point OnInitialPosition([[maybe_unused]] int16_t sm_width, [[maybe_unused]] int16_t sm_height, [[maybe_unused]] int window_number) override
 	{
 		Point pt = { 0, _screen.height - sm_height - FindWindowById(WC_STATUS_BAR, 0)->height };
 		return pt;
 	}
 
-	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
+	void SetStringParameters(WidgetID widget) const override
 	{
 		if (widget != WID_NC_DESTINATION) return;
 
 		if (this->dtype == DESTTYPE_CLIENT) {
 			SetDParamStr(0, NetworkClientInfo::GetByClientID((ClientID)this->dest)->client_name);
 		}
-		Dimension d = GetStringBoundingBox(this->dest_string);
-		d.width  += WD_FRAMERECT_LEFT + WD_FRAMERECT_RIGHT;
-		d.height += WD_FRAMERECT_TOP + WD_FRAMERECT_BOTTOM;
-		*size = maxdim(*size, d);
 	}
 
-	virtual void DrawWidget(const Rect &r, int widget) const
-	{
-		if (widget != WID_NC_DESTINATION) return;
-
-		if (this->dtype == DESTTYPE_CLIENT) {
-			SetDParamStr(0, NetworkClientInfo::GetByClientID((ClientID)this->dest)->client_name);
-		}
-		DrawString(r.left + WD_FRAMERECT_LEFT, r.right - WD_FRAMERECT_RIGHT, r.top + WD_FRAMERECT_TOP, this->dest_string, TC_BLACK, SA_RIGHT);
-	}
-
-	virtual void OnClick(Point pt, int widget, int click_count)
+	void OnClick([[maybe_unused]] Point pt, WidgetID widget, [[maybe_unused]] int click_count) override
 	{
 		switch (widget) {
-			/* Send */
-			case WID_NC_SENDBUTTON: SendChat(this->text.buf, this->dtype, this->dest);
-				/* FALL THROUGH */
-			case WID_NC_CLOSE: /* Cancel */ delete this; break;
+			case WID_NC_SENDBUTTON: /* Send */
+				SendChat(this->message_editbox.text.buf, this->dtype, this->dest);
+				[[fallthrough]];
+
+			case WID_NC_CLOSE: /* Cancel */
+				this->Close();
+				break;
 		}
 	}
 
-	virtual void OnMouseLoop()
-	{
-		this->HandleEditBox(WID_NC_TEXTBOX);
-	}
-
-	virtual EventState OnKeyPress(uint16 key, uint16 keycode)
+	EventState OnKeyPress([[maybe_unused]] char32_t key, uint16_t keycode) override
 	{
 		EventState state = ES_NOT_HANDLED;
 		if (keycode == WKC_TAB) {
 			ChatTabCompletion();
 			state = ES_HANDLED;
-		} else {
-			_chat_tab_completion_active = false;
-			switch (this->HandleEditBoxKey(WID_NC_TEXTBOX, key, keycode, state)) {
-				default: NOT_REACHED();
-				case HEBR_EDITING: {
-					Window *osk = FindWindowById(WC_OSK, 0);
-					if (osk != NULL && osk->parent == this) osk->InvalidateData();
-					break;
-				}
-				case HEBR_CONFIRM:
-					SendChat(this->text.buf, this->dtype, this->dest);
-					/* FALL THROUGH */
-				case HEBR_CANCEL: delete this; break;
-				case HEBR_NOT_FOCUSED: break;
-			}
 		}
 		return state;
 	}
 
-	virtual void OnOpenOSKWindow(int wid)
+	void OnEditboxChanged(WidgetID widget) override
 	{
-		ShowOnScreenKeyboard(this, wid, WID_NC_CLOSE, WID_NC_SENDBUTTON);
+		if (widget == WID_NC_TEXTBOX) {
+			_chat_tab_completion_active = false;
+		}
 	}
 
 	/**
@@ -548,19 +484,19 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 	 * @param data Information about the changed data.
 	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
 	 */
-	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
+	void OnInvalidateData([[maybe_unused]] int data = 0, [[maybe_unused]] bool gui_scope = true) override
 	{
-		if (data == this->dest) delete this;
+		if (data == this->dest) this->Close();
 	}
 };
 
 /** The widgets of the chat window. */
-static const NWidgetPart _nested_chat_window_widgets[] = {
+static constexpr NWidgetPart _nested_chat_window_widgets[] = {
 	NWidget(NWID_HORIZONTAL),
 		NWidget(WWT_CLOSEBOX, COLOUR_GREY, WID_NC_CLOSE),
 		NWidget(WWT_PANEL, COLOUR_GREY, WID_NC_BACKGROUND),
 			NWidget(NWID_HORIZONTAL),
-				NWidget(WWT_TEXT, COLOUR_GREY, WID_NC_DESTINATION), SetMinimalSize(62, 12), SetPadding(1, 0, 1, 0), SetDataTip(STR_NULL, STR_NULL),
+				NWidget(WWT_TEXT, COLOUR_GREY, WID_NC_DESTINATION), SetMinimalSize(62, 12), SetPadding(1, 0, 1, 0), SetAlignment(SA_VERT_CENTER | SA_RIGHT), SetDataTip(STR_NULL, STR_NULL),
 				NWidget(WWT_EDITBOX, COLOUR_GREY, WID_NC_TEXTBOX), SetMinimalSize(100, 12), SetPadding(1, 0, 1, 0), SetResize(1, 0),
 																	SetDataTip(STR_NETWORK_CHAT_OSKTITLE, STR_NULL),
 				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_NC_SENDBUTTON), SetMinimalSize(62, 12), SetPadding(1, 0, 1, 0), SetDataTip(STR_NETWORK_CHAT_SEND, STR_NULL),
@@ -570,11 +506,11 @@ static const NWidgetPart _nested_chat_window_widgets[] = {
 };
 
 /** The description of the chat window. */
-static const WindowDesc _chat_window_desc(
-	WDP_MANUAL, 640, 14, // x, y, width, height
+static WindowDesc _chat_window_desc(__FILE__, __LINE__,
+	WDP_MANUAL, nullptr, 0, 0,
 	WC_SEND_NETWORK_MSG, WC_NONE,
 	0,
-	_nested_chat_window_widgets, lengthof(_nested_chat_window_widgets)
+	std::begin(_nested_chat_window_widgets), std::end(_nested_chat_window_widgets)
 );
 
 
@@ -585,8 +521,6 @@ static const WindowDesc _chat_window_desc(
  */
 void ShowNetworkChatQueryWindow(DestType type, int dest)
 {
-	DeleteWindowByClass(WC_SEND_NETWORK_MSG);
+	CloseWindowByClass(WC_SEND_NETWORK_MSG);
 	new NetworkChatWindow(&_chat_window_desc, type, dest);
 }
-
-#endif /* ENABLE_NETWORK */
