@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -13,8 +11,6 @@
 #define SCRIPT_CONTROLLER_HPP
 
 #include "script_types.hpp"
-#include "../../core/string_compare_type.hpp"
-#include <map>
 
 /**
  * The Controller, the class each Script should extend. It creates the Script,
@@ -25,7 +21,6 @@
  *  script that matches to the specified version as close as possible. It tries
  *  (from first to last, stopping as soon as the attempt succeeds)
  *
- *  - load the exact same version of the same script,
  *  - load the latest version of the same script that supports loading data from
  *    the saved version (the version of saved data must be equal or greater
  *    than ScriptInfo::MinVersionToLoad),
@@ -42,7 +37,7 @@
  *   data from the loaded game.
  * - Finally, #Start is called to start execution of the script.
  *
- * See also http://wiki.openttd.org/AI:Save/Load for more details.
+ * See also https://wiki.openttd.org/en/Development/Script/Save%20and%20Load for more details.
  *
  * @api ai game
  */
@@ -51,17 +46,14 @@ class ScriptController {
 	friend class ScriptInstance;
 
 public:
+#ifndef DOXYGEN_API
 	/**
 	 * Initializer of the ScriptController.
 	 * @param company The company this Script is normally serving.
 	 */
 	ScriptController(CompanyID company);
 
-	/**
-	 * Destructor of the ScriptController.
-	 */
-	~ScriptController();
-
+#else
 	/**
 	 * This function is called to start your script. Your script starts here. If you
 	 *   return from this function, your script dies, so make sure that doesn't
@@ -70,7 +62,6 @@ public:
 	 */
 	void Start();
 
-#ifdef DOXYGEN_API
 	/**
 	 * Save the state of the script.
 	 *
@@ -94,7 +85,7 @@ public:
 	 *   notified of the call. To avoid race-conditions between #Save and the
 	 *   other script code, change variables directly after a #Sleep, it is
 	 *   very unlikely, to get interrupted at that point in the execution.
-	 * See also http://wiki.openttd.org/AI:Save/Load for more details.
+	 * See also https://wiki.openttd.org/en/Development/Script/Save%20and%20Load for more details.
 	 *
 	 * @note No other information is saved than the table returned by #Save.
 	 *   For example all pending events are lost as soon as the game is loaded.
@@ -133,16 +124,25 @@ public:
 	 * @param name The name of the setting.
 	 * @return the value for the setting, or -1 if the setting is not known.
 	 */
-	static int GetSetting(const char *name);
+	static int GetSetting(const std::string &name);
 
 	/**
 	 * Get the OpenTTD version of this executable. The version is formatted
 	 * with the bits having the following meaning:
-	 * 28-31 major version
-	 * 24-27 minor version
-	 * 20-23 build
+	 * 24-31 major version + 16.
+	 * 20-23 minor version.
 	 *    19 1 if it is a release, 0 if it is not.
 	 *  0-18 revision number; 0 when the revision is unknown.
+	 * You have to subtract 16 from the major version to get the correct
+	 * value.
+	 *
+	 * Prior to OpenTTD 12, the bits have the following meaning:
+	 * 28-31 major version.
+	 * 24-27 minor version.
+	 * 20-23 build.
+	 *    19 1 if it is a release, 0 if it is not.
+	 *  0-18 revision number; 0 when the revision is unknown.
+	 *
 	 * @return The version in newgrf format.
 	 */
 	static uint GetVersion();
@@ -180,7 +180,7 @@ public:
 	 * @note gui.ai_developer_tools setting must be enabled or the break is
 	 * ignored.
 	 */
-	static void Break(const char* message);
+	static void Break(const std::string &message);
 
 	/**
 	 * When Squirrel triggers a print, this function is called.
@@ -189,7 +189,7 @@ public:
 	 * @param message The message Squirrel logged.
 	 * @note Use ScriptLog.Info/Warning/Error instead of 'print'.
 	 */
-	static void Print(bool error_msg, const char *message);
+	static void Print(bool error_msg, const std::string &message);
 
 	/**
 	 * Import a library.
@@ -200,10 +200,10 @@ public:
 	 * @return The loaded library object. If class_name is set, it is also available (under the scope of the import) under that name.
 	 * @note This command can be called from the global space, and does not need an instance.
 	 */
-	static HSQOBJECT Import(const char *library, const char *class_name, int version);
+	static HSQOBJECT Import(const std::string &library, const std::string &class_name, int version);
 
 private:
-	typedef std::map<const char *, const char *, StringCompare> LoadedLibraryList; ///< The type for loaded libraries.
+	typedef std::map<std::string, std::string, CaseInsensitiveComparator> LoadedLibraryList; ///< The type for loaded libraries.
 
 	uint ticks;                       ///< The amount of ticks we're sleeping.
 	LoadedLibraryList loaded_library; ///< The libraries we loaded.
