@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -10,7 +8,10 @@
 /** @file strgen_base.cpp Tool to create computer readable (stand-alone) translation files. */
 
 #include "../stdafx.h"
+#include "../core/alloc_func.hpp"
 #include "../core/endian_func.hpp"
+#include "../core/mem_func.hpp"
+#include "../error_func.h"
 #include "../string_func.h"
 #include "../table/control_codes.h"
 
@@ -37,20 +38,12 @@ static const CmdStruct *ParseCommandString(const char **str, char *param, int *a
  * Create a new case.
  * @param caseidx The index of the case.
  * @param string  The translation of the case.
- * @param next    The next chained case.
  */
-Case::Case(int caseidx, const char *string, Case *next) :
-		caseidx(caseidx), string(stredup(string)), next(next)
+Case::Case(int caseidx, const std::string &string) :
+		caseidx(caseidx), string(string)
 {
 }
 
-/** Free everything we allocated. */
-Case::~Case()
-{
-	free(this->string);
-	delete this->next;
-}
-
 /**
  * Create a new string.
  * @param name    The name of the string.
@@ -58,29 +51,16 @@ Case::~Case()
  * @param index   The index in the string table.
  * @param line    The line this string was found on.
  */
-LangString::LangString(const char *name, const char *english, int index, int line) :
-		name(stredup(name)), english(stredup(english)), translated(NULL),
-		hash_next(0), index(index), line(line), translated_case(NULL)
+LangString::LangString(const std::string &name, const std::string &english, size_t index, int line) :
+		name(name), english(english), index(index), line(line)
 {
 }
 
-/** Free everything we allocated. */
-LangString::~LangString()
-{
-	free(this->name);
-	free(this->english);
-	free(this->translated);
-	delete this->translated_case;
-}
-
 /** Free all data related to the translation. */
 void LangString::FreeTranslation()
 {
-	free(this->translated);
-	this->translated = NULL;
-
-	delete this->translated_case;
-	this->translated_case = NULL;
+	this->translated.clear();
+	this->translated_cases.clear();
 }
 
 /**
@@ -89,70 +69,41 @@ void LangString::FreeTranslation()
  */
 StringData::StringData(size_t tabs) : tabs(tabs), max_strings(tabs * TAB_SIZE)
 {
-	this->strings = CallocT<LangString *>(max_strings);
-	this->hash_heads = CallocT<uint16>(max_strings);
+	this->strings.resize(max_strings);
 	this->next_string_id = 0;
 }
 
-/** Free everything we allocated. */
-StringData::~StringData()
-{
-	for (size_t i = 0; i < this->max_strings; i++) delete this->strings[i];
-	free(this->strings);
-	free(this->hash_heads);
-}
-
 /** Free all data related to the translation. */
 void StringData::FreeTranslation()
 {
 	for (size_t i = 0; i < this->max_strings; i++) {
-		LangString *ls = this->strings[i];
-		if (ls != NULL) ls->FreeTranslation();
+		LangString *ls = this->strings[i].get();
+		if (ls != nullptr) ls->FreeTranslation();
 	}
 }
 
-/**
- * Create a hash of the string for finding them back quickly.
- * @param s The string to hash.
- * @return The hashed string.
- */
-uint StringData::HashStr(const char *s) const
-{
-	uint hash = 0;
-	for (; *s != '\0'; s++) hash = ROL(hash, 3) ^ *s;
-	return hash % this->max_strings;
-}
-
 /**
  * Add a newly created LangString.
  * @param s  The name of the string.
  * @param ls The string to add.
  */
-void StringData::Add(const char *s, LangString *ls)
+void StringData::Add(std::unique_ptr<LangString> ls)
 {
-	uint hash = this->HashStr(s);
-	ls->hash_next = this->hash_heads[hash];
-	/* Off-by-one for hash find. */
-	this->hash_heads[hash] = ls->index + 1;
-	this->strings[ls->index] = ls;
+	this->name_to_string[ls->name] = ls.get();
+	this->strings[ls->index].swap(ls);
 }
 
 /**
  * Find a LangString based on the string name.
  * @param s The string name to search on.
- * @return The LangString or NULL if it is not known.
+ * @return The LangString or nullptr if it is not known.
  */
-LangString *StringData::Find(const char *s)
+LangString *StringData::Find(const std::string_view s)
 {
-	int idx = this->hash_heads[this->HashStr(s)];
+	auto it = this->name_to_string.find(s);
+	if (it == this->name_to_string.end()) return nullptr;
 
-	while (--idx >= 0) {
-		LangString *ls = this->strings[idx];
-
-		if (strcmp(ls->name, s) == 0) return ls;
-		idx = ls->hash_next;
-	}
-	return NULL;
+	return it->second;
 }
 
 /**
@@ -164,7 +115,7 @@ LangString *StringData::Find(const char *s)
 uint StringData::VersionHashStr(uint hash, const char *s) const
 {
 	for (; *s != '\0'; s++) {
-		hash = ROL(hash, 3) ^ *s;
+		hash = std::rotl(hash, 3) ^ *s;
 		hash = (hash & 1 ? hash >> 1 ^ 0xDEADBEEF : hash >> 1);
 	}
 	return hash;
@@ -179,22 +130,22 @@ uint StringData::Version() const
 	uint hash = 0;
 
 	for (size_t i = 0; i < this->max_strings; i++) {
-		const LangString *ls = this->strings[i];
+		const LangString *ls = this->strings[i].get();
 
-		if (ls != NULL) {
+		if (ls != nullptr) {
 			const CmdStruct *cs;
 			const char *s;
 			char buf[MAX_COMMAND_PARAM_SIZE];
 			int argno;
 			int casei;
 
-			s = ls->name;
+			s = ls->name.c_str();
 			hash ^= i * 0x717239;
 			hash = (hash & 1 ? hash >> 1 ^ 0xDEADBEEF : hash >> 1);
 			hash = this->VersionHashStr(hash, s + 1);
 
-			s = ls->english;
-			while ((cs = ParseCommandString(&s, buf, &argno, &casei)) != NULL) {
+			s = ls->english.c_str();
+			while ((cs = ParseCommandString(&s, buf, &argno, &casei)) != nullptr) {
 				if (cs->flags & C_DONTCOUNT) continue;
 
 				hash ^= (cs - _cmd_structs) * 0x1234567;
@@ -213,67 +164,56 @@ uint StringData::Version() const
 uint StringData::CountInUse(uint tab) const
 {
 	int i;
-	for (i = TAB_SIZE; --i >= 0;) if (this->strings[(tab * TAB_SIZE) + i] != NULL) break;
+	for (i = TAB_SIZE; --i >= 0;) if (this->strings[(tab * TAB_SIZE) + i] != nullptr) break;
 	return i + 1;
 }
 
 static const char *_cur_ident;
 
-struct CmdPair {
-	const CmdStruct *a;
-	const char *v;
-};
-
-struct ParsedCommandStruct {
-	uint np;
-	CmdPair pairs[32];
-	const CmdStruct *cmd[32]; // ordered by param #
-};
-
 /* Used when generating some advanced commands. */
 static ParsedCommandStruct _cur_pcs;
 static int _cur_argidx;
 
 /** The buffer for writing a single string. */
-struct Buffer : SmallVector<byte, 256> {
+struct Buffer : std::vector<byte> {
 	/**
 	 * Convenience method for adding a byte.
 	 * @param value The value to add.
 	 */
 	void AppendByte(byte value)
 	{
-		*this->Append() = value;
+		this->push_back(value);
 	}
 
 	/**
 	 * Add an Unicode character encoded in UTF-8 to the buffer.
 	 * @param value The character to add.
 	 */
-	void AppendUtf8(uint32 value)
+	void AppendUtf8(uint32_t value)
 	{
 		if (value < 0x80) {
-			*this->Append() = value;
+			this->push_back(value);
 		} else if (value < 0x800) {
-			*this->Append() = 0xC0 + GB(value,  6, 5);
-			*this->Append() = 0x80 + GB(value,  0, 6);
+			this->push_back(0xC0 + GB(value,  6, 5));
+			this->push_back(0x80 + GB(value,  0, 6));
 		} else if (value < 0x10000) {
-			*this->Append() = 0xE0 + GB(value, 12, 4);
-			*this->Append() = 0x80 + GB(value,  6, 6);
-			*this->Append() = 0x80 + GB(value,  0, 6);
+			this->push_back(0xE0 + GB(value, 12, 4));
+			this->push_back(0x80 + GB(value,  6, 6));
+			this->push_back(0x80 + GB(value,  0, 6));
 		} else if (value < 0x110000) {
-			*this->Append() = 0xF0 + GB(value, 18, 3);
-			*this->Append() = 0x80 + GB(value, 12, 6);
-			*this->Append() = 0x80 + GB(value,  6, 6);
-			*this->Append() = 0x80 + GB(value,  0, 6);
+			this->push_back(0xF0 + GB(value, 18, 3));
+			this->push_back(0x80 + GB(value, 12, 6));
+			this->push_back(0x80 + GB(value,  6, 6));
+			this->push_back(0x80 + GB(value,  0, 6));
 		} else {
-			strgen_warning("Invalid unicode value U+0x%X", value);
+			StrgenWarning("Invalid unicode value U+0x{:X}", value);
 		}
 	}
 };
 
 size_t Utf8Validate(const char *s)
 {
-	uint32 c;
+	uint32_t c;
 
 	if (!HasBit(s[0], 7)) {
 		/* 1 byte */
@@ -298,7 +238,7 @@ size_t Utf8Validate(const char *s)
 
 void EmitSingleChar(Buffer *buffer, char *buf, int value)
 {
-	if (*buf != '\0') strgen_warning("Ignoring trailing letters in command");
+	if (*buf != '\0') StrgenWarning("Ignoring trailing letters in command");
 	buffer->AppendUtf8(value);
 }
 
@@ -320,30 +260,30 @@ bool ParseRelNum(char **buf, int *value, int *offset)
 		rel = true;
 		s++;
 	}
-	int v = strtol(s, &end, 0);
+	int v = std::strtol(s, &end, 0);
 	if (end == s) return false;
 	if (rel || v < 0) {
 		*value += v;
 	} else {
 		*value = v;
 	}
-	if (offset != NULL && *end == ':') {
+	if (offset != nullptr && *end == ':') {
 		/* Take the Nth within */
 		s = end + 1;
-		*offset = strtol(s, &end, 0);
+		*offset = std::strtol(s, &end, 0);
 		if (end == s) return false;
 	}
 	*buf = end;
 	return true;
 }
 
-/* Parse out the next word, or NULL */
+/* Parse out the next word, or nullptr */
 char *ParseWord(char **buf)
 {
 	char *s = *buf, *r;
 
 	while (*s == ' ' || *s == '\t') s++;
-	if (*s == '\0') return NULL;
+	if (*s == '\0') return nullptr;
 
 	if (*s == '"') {
 		r = ++s;
@@ -375,7 +315,7 @@ char *ParseWord(char **buf)
 /* Forward declaration */
 static int TranslateArgumentIdx(int arg, int offset = 0);
 
-static void EmitWordList(Buffer *buffer, const char * const *words, uint nw)
+static void EmitWordList(Buffer *buffer, const std::vector<const char *> &words, uint nw)
 {
 	buffer->AppendByte(nw);
 	for (uint i = 0; i < nw; i++) buffer->AppendByte((byte)strlen(words[i]) + 1);
@@ -385,22 +325,22 @@ static void EmitWordList(Buffer *buffer, const char * const *words, uint nw)
 	}
 }
 
-void EmitPlural(Buffer *buffer, char *buf, int value)
+void EmitPlural(Buffer *buffer, char *buf, int)
 {
 	int argidx = _cur_argidx;
 	int offset = -1;
 	int expected = _plural_forms[_lang.plural_form].plural_count;
-	const char **words = AllocaM(const char *, max(expected, MAX_PLURALS));
+	std::vector<const char *> words(std::max(expected, MAX_PLURALS), nullptr);
 	int nw = 0;
 
 	/* Parse out the number, if one exists. Otherwise default to prev arg. */
 	if (!ParseRelNum(&buf, &argidx, &offset)) argidx--;
 
-	const CmdStruct *cmd = _cur_pcs.cmd[argidx];
+	const CmdStruct *cmd = _cur_pcs.consuming_commands[argidx];
 	if (offset == -1) {
 		/* Use default offset */
-		if (cmd == NULL || cmd->default_plural_offset < 0) {
-			strgen_fatal("Command '%s' has no (default) plural position", cmd == NULL ? "<empty>" : cmd->cmd);
+		if (cmd == nullptr || cmd->default_plural_offset < 0) {
+			StrgenFatal("Command '{}' has no (default) plural position", cmd == nullptr ? "<empty>" : cmd->cmd);
 		}
 		offset = cmd->default_plural_offset;
 	}
@@ -408,19 +348,19 @@ void EmitPlural(Buffer *buffer, char *buf, int value)
 	/* Parse each string */
 	for (nw = 0; nw < MAX_PLURALS; nw++) {
 		words[nw] = ParseWord(&buf);
-		if (words[nw] == NULL) break;
+		if (words[nw] == nullptr) break;
 	}
 
 	if (nw == 0) {
-		strgen_fatal("%s: No plural words", _cur_ident);
+		StrgenFatal("{}: No plural words", _cur_ident);
 	}
 
 	if (expected != nw) {
 		if (_translated) {
-			strgen_fatal("%s: Invalid number of plural forms. Expecting %d, found %d.", _cur_ident,
+			StrgenFatal("{}: Invalid number of plural forms. Expecting {}, found {}.", _cur_ident,
 				expected, nw);
 		} else {
-			if ((_show_todo & 2) != 0) strgen_warning("'%s' is untranslated. Tweaking english string to allow compilation for plural forms", _cur_ident);
+			if ((_show_todo & 2) != 0) StrgenWarning("'{}' is untranslated. Tweaking english string to allow compilation for plural forms", _cur_ident);
 			if (nw > expected) {
 				nw = expected;
 			} else {
@@ -437,8 +377,7 @@ void EmitPlural(Buffer *buffer, char *buf, int value)
 	EmitWordList(buffer, words, nw);
 }
 
-
-void EmitGender(Buffer *buffer, char *buf, int value)
+void EmitGender(Buffer *buffer, char *buf, int)
 {
 	int argidx = _cur_argidx;
 	int offset = 0;
@@ -449,28 +388,28 @@ void EmitGender(Buffer *buffer, char *buf, int value)
 
 		/* This is a {G=DER} command */
 		nw = _lang.GetGenderIndex(buf);
-		if (nw >= MAX_NUM_GENDERS) strgen_fatal("G argument '%s' invalid", buf);
+		if (nw >= MAX_NUM_GENDERS) StrgenFatal("G argument '{}' invalid", buf);
 
 		/* now nw contains the gender index */
 		buffer->AppendUtf8(SCC_GENDER_INDEX);
 		buffer->AppendByte(nw);
 	} else {
-		const char *words[MAX_NUM_GENDERS];
+		std::vector<const char *> words(MAX_NUM_GENDERS, nullptr);
 
 		/* This is a {G 0 foo bar two} command.
 		 * If no relative number exists, default to +0 */
-		if (!ParseRelNum(&buf, &argidx, &offset)) {}
+		ParseRelNum(&buf, &argidx, &offset);
 
-		const CmdStruct *cmd = _cur_pcs.cmd[argidx];
-		if (cmd == NULL || (cmd->flags & C_GENDER) == 0) {
-			strgen_fatal("Command '%s' can't have a gender", cmd == NULL ? "<empty>" : cmd->cmd);
+		const CmdStruct *cmd = _cur_pcs.consuming_commands[argidx];
+		if (cmd == nullptr || (cmd->flags & C_GENDER) == 0) {
+			StrgenFatal("Command '{}' can't have a gender", cmd == nullptr ? "<empty>" : cmd->cmd);
 		}
 
 		for (nw = 0; nw < MAX_NUM_GENDERS; nw++) {
 			words[nw] = ParseWord(&buf);
-			if (words[nw] == NULL) break;
+			if (words[nw] == nullptr) break;
 		}
-		if (nw != _lang.num_genders) strgen_fatal("Bad # of arguments for gender command");
+		if (nw != _lang.num_genders) StrgenFatal("Bad # of arguments for gender command");
 
 		assert(IsInsideBS(cmd->value, SCC_CONTROL_START, UINT8_MAX));
 		buffer->AppendUtf8(SCC_GENDER_LIST);
@@ -484,24 +423,24 @@ static const CmdStruct *FindCmd(const char *s, int len)
 	for (const CmdStruct *cs = _cmd_structs; cs != endof(_cmd_structs); cs++) {
 		if (strncmp(cs->cmd, s, len) == 0 && cs->cmd[len] == '\0') return cs;
 	}
-	return NULL;
+	return nullptr;
 }
 
 static uint ResolveCaseName(const char *str, size_t len)
 {
 	/* First get a clean copy of only the case name, then resolve it. */
 	char case_str[CASE_GENDER_LEN];
-	len = min(lengthof(case_str) - 1, len);
+	len = std::min(lengthof(case_str) - 1, len);
 	memcpy(case_str, str, len);
 	case_str[len] = '\0';
 
-	uint8 case_idx = _lang.GetCaseIndex(case_str);
-	if (case_idx >= MAX_NUM_CASES) strgen_fatal("Invalid case-name '%s'", case_str);
+	uint8_t case_idx = _lang.GetCaseIndex(case_str);
+	if (case_idx >= MAX_NUM_CASES) StrgenFatal("Invalid case-name '{}'", case_str);
 	return case_idx + 1;
 }
 
 
-/* returns NULL on eof
+/* returns nullptr on eof
  * else returns command struct */
 static const CmdStruct *ParseCommandString(const char **str, char *param, int *argno, int *casei)
 {
@@ -513,15 +452,15 @@ static const CmdStruct *ParseCommandString(const char **str, char *param, int *a
 
 	/* Scan to the next command, exit if there's no next command. */
 	for (; *s != '{'; s++) {
-		if (*s == '\0') return NULL;
+		if (*s == '\0') return nullptr;
 	}
 	s++; // Skip past the {
 
 	if (*s >= '0' && *s <= '9') {
 		char *end;
 
-		*argno = strtoul(s, &end, 0);
-		if (*end != ':') strgen_fatal("missing arg #");
+		*argno = std::strtoul(s, &end, 0);
+		if (*end != ':') StrgenFatal("missing arg #");
 		s = end + 1;
 	}
 
@@ -532,16 +471,17 @@ static const CmdStruct *ParseCommandString(const char **str, char *param, int *a
 	} while (c != '}' && c != ' ' && c != '=' && c != '.' && c != 0);
 
 	const CmdStruct *cmd = FindCmd(start, s - start - 1);
-	if (cmd == NULL) {
-		strgen_error("Undefined command '%.*s'", (int)(s - start - 1), start);
-		return NULL;
+	if (cmd == nullptr) {
+		std::string command(start, s - start - 1);
+		StrgenError("Undefined command '{}'", command);
+		return nullptr;
 	}
 
 	if (c == '.') {
 		const char *casep = s;
 
 		if (!(cmd->flags & C_CASE)) {
-			strgen_fatal("Command '%s' can't have a case", cmd->cmd);
+			StrgenFatal("Command '{}' can't have a case", cmd->cmd);
 		}
 
 		do {
@@ -551,8 +491,8 @@ static const CmdStruct *ParseCommandString(const char **str, char *param, int *a
 	}
 
 	if (c == '\0') {
-		strgen_error("Missing } from command '%s'", start);
-		return NULL;
+		StrgenError("Missing }} from command '{}'", start);
+		return nullptr;
 	}
 
 
@@ -564,10 +504,10 @@ static const CmdStruct *ParseCommandString(const char **str, char *param, int *a
 			c = *s++;
 			if (c == '}') break;
 			if (c == '\0') {
-				strgen_error("Missing } from command '%s'", start);
-				return NULL;
+				StrgenError("Missing }} from command '{}'", start);
+				return nullptr;
 			}
-			if (s - start == MAX_COMMAND_PARAM_SIZE) error("param command too long");
+			if (s - start == MAX_COMMAND_PARAM_SIZE) FatalError("param command too long");
 			*param++ = c;
 		}
 	}
@@ -585,54 +525,47 @@ static const CmdStruct *ParseCommandString(const char **str, char *param, int *a
  * @param master      Are we reading the master file?
  * @param translation Are we reading a translation?
  */
-StringReader::StringReader(StringData &data, const char *file, bool master, bool translation) :
-		data(data), file(stredup(file)), master(master), translation(translation)
+StringReader::StringReader(StringData &data, const std::string &file, bool master, bool translation) :
+		data(data), file(file), master(master), translation(translation)
 {
 }
 
-/** Make sure the right reader gets freed. */
-StringReader::~StringReader()
-{
-	free(file);
-}
-
-static void ExtractCommandString(ParsedCommandStruct *p, const char *s, bool warnings)
+ParsedCommandStruct ExtractCommandString(const char *s, bool)
 {
 	char param[MAX_COMMAND_PARAM_SIZE];
 	int argno;
 	int argidx = 0;
 	int casei;
 
-	memset(p, 0, sizeof(*p));
+	ParsedCommandStruct p;
 
 	for (;;) {
 		/* read until next command from a. */
 		const CmdStruct *ar = ParseCommandString(&s, param, &argno, &casei);
 
-		if (ar == NULL) break;
+		if (ar == nullptr) break;
 
 		/* Sanity checking */
-		if (argno != -1 && ar->consumes == 0) strgen_fatal("Non consumer param can't have a paramindex");
+		if (argno != -1 && ar->consumes == 0) StrgenFatal("Non consumer param can't have a paramindex");
 
 		if (ar->consumes) {
 			if (argno != -1) argidx = argno;
-			if (argidx < 0 || (uint)argidx >= lengthof(p->cmd)) strgen_fatal("invalid param idx %d", argidx);
-			if (p->cmd[argidx] != NULL && p->cmd[argidx] != ar) strgen_fatal("duplicate param idx %d", argidx);
+			if (argidx < 0 || (uint)argidx >= p.consuming_commands.max_size()) StrgenFatal("invalid param idx {}", argidx);
+			if (p.consuming_commands[argidx] != nullptr && p.consuming_commands[argidx] != ar) StrgenFatal("duplicate param idx {}", argidx);
 
-			p->cmd[argidx++] = ar;
+			p.consuming_commands[argidx++] = ar;
 		} else if (!(ar->flags & C_DONTCOUNT)) { // Ignore some of them
-			if (p->np >= lengthof(p->pairs)) strgen_fatal("too many commands in string, max " PRINTF_SIZE, lengthof(p->pairs));
-			p->pairs[p->np].a = ar;
-			p->pairs[p->np].v = param[0] != '\0' ? stredup(param) : "";
-			p->np++;
+			p.non_consuming_commands.emplace_back(CmdPair{ar, param});
 		}
 	}
+
+	return p;
 }
 
 
-static const CmdStruct *TranslateCmdForCompare(const CmdStruct *a)
+const CmdStruct *TranslateCmdForCompare(const CmdStruct *a)
 {
-	if (a == NULL) return NULL;
+	if (a == nullptr) return nullptr;
 
 	if (strcmp(a->cmd, "STRING1") == 0 ||
 			strcmp(a->cmd, "STRING2") == 0 ||
@@ -649,7 +582,7 @@ static const CmdStruct *TranslateCmdForCompare(const CmdStruct *a)
 }
 
 
-static bool CheckCommandsMatch(char *a, char *b, const char *name)
+static bool CheckCommandsMatch(const char *a, const char *b, const char *name)
 {
 	/* If we're not translating, i.e. we're compiling the base language,
 	 * it is pointless to do all these checks as it'll always be correct.
@@ -657,45 +590,42 @@ static bool CheckCommandsMatch(char *a, char *b, const char *name)
 	 */
 	if (!_translation) return true;
 
-	ParsedCommandStruct templ;
-	ParsedCommandStruct lang;
 	bool result = true;
 
-	ExtractCommandString(&templ, b, true);
-	ExtractCommandString(&lang, a, true);
+	ParsedCommandStruct templ = ExtractCommandString(b, true);
+	ParsedCommandStruct lang = ExtractCommandString(a, true);
 
 	/* For each string in templ, see if we find it in lang */
-	if (templ.np != lang.np) {
-		strgen_warning("%s: template string and language string have a different # of commands", name);
+	if (templ.non_consuming_commands.max_size() != lang.non_consuming_commands.max_size()) {
+		StrgenWarning("{}: template string and language string have a different # of commands", name);
 		result = false;
 	}
 
-	for (uint i = 0; i < templ.np; i++) {
+	for (auto &templ_nc : templ.non_consuming_commands) {
 		/* see if we find it in lang, and zero it out */
 		bool found = false;
-		for (uint j = 0; j < lang.np; j++) {
-			if (templ.pairs[i].a == lang.pairs[j].a &&
-					strcmp(templ.pairs[i].v, lang.pairs[j].v) == 0) {
+		for (auto &lang_nc : lang.non_consuming_commands) {
+			if (templ_nc.cmd == lang_nc.cmd && templ_nc.param == lang_nc.param) {
 				/* it was found in both. zero it out from lang so we don't find it again */
-				lang.pairs[j].a = NULL;
+				lang_nc.cmd = nullptr;
 				found = true;
 				break;
 			}
 		}
 
 		if (!found) {
-			strgen_warning("%s: command '%s' exists in template file but not in language file", name, templ.pairs[i].a->cmd);
+			StrgenWarning("{}: command '{}' exists in template file but not in language file", name, templ_nc.cmd->cmd);
 			result = false;
 		}
 	}
 
 	/* if we reach here, all non consumer commands match up.
 	 * Check if the non consumer commands match up also. */
-	for (uint i = 0; i < lengthof(templ.cmd); i++) {
-		if (TranslateCmdForCompare(templ.cmd[i]) != lang.cmd[i]) {
-			strgen_warning("%s: Param idx #%d '%s' doesn't match with template command '%s'", name, i,
-				lang.cmd[i]  == NULL ? "<empty>" : TranslateCmdForCompare(lang.cmd[i])->cmd,
-				templ.cmd[i] == NULL ? "<empty>" : templ.cmd[i]->cmd);
+	for (uint i = 0; i < templ.consuming_commands.max_size(); i++) {
+		if (TranslateCmdForCompare(templ.consuming_commands[i]) != lang.consuming_commands[i]) {
+			StrgenWarning("{}: Param idx #{} '{}' doesn't match with template command '{}'", name, i,
+				lang.consuming_commands[i]  == nullptr ? "<empty>" : TranslateCmdForCompare(lang.consuming_commands[i])->cmd,
+				templ.consuming_commands[i] == nullptr ? "<empty>" : templ.consuming_commands[i]->cmd);
 			result = false;
 		}
 	}
@@ -714,8 +644,8 @@ void StringReader::HandleString(char *str)
 	if (*str == ';' || *str == ' ' || *str == '\0') return;
 
 	char *s = strchr(str, ':');
-	if (s == NULL) {
-		strgen_error("Line has no ':' delimiter");
+	if (s == nullptr) {
+		StrgenError("Line has no ':' delimiter");
 		return;
 	}
 
@@ -730,15 +660,15 @@ void StringReader::HandleString(char *str)
 	const char *tmp;
 	for (tmp = s; *tmp != '\0';) {
 		size_t len = Utf8Validate(tmp);
-		if (len == 0) strgen_fatal("Invalid UTF-8 sequence in '%s'", s);
+		if (len == 0) StrgenFatal("Invalid UTF-8 sequence in '{}'", s);
 
-		WChar c;
+		char32_t c;
 		Utf8Decode(&c, tmp);
 		if (c <= 0x001F || // ASCII control character range
 				c == 0x200B || // Zero width space
 				(c >= 0xE000 && c <= 0xF8FF) || // Private range
 				(c >= 0xFFF0 && c <= 0xFFFF)) { // Specials range
-			strgen_fatal("Unwanted UTF-8 character U+%04X in sequence '%s'", c, s);
+			StrgenFatal("Unwanted UTF-8 character U+{:04X} in sequence '{}'", (int)c, s);
 		}
 
 		tmp += len;
@@ -747,47 +677,47 @@ void StringReader::HandleString(char *str)
 	/* Check if the string has a case..
 	 * The syntax for cases is IDENTNAME.case */
 	char *casep = strchr(str, '.');
-	if (casep != NULL) *casep++ = '\0';
+	if (casep != nullptr) *casep++ = '\0';
 
 	/* Check if this string already exists.. */
 	LangString *ent = this->data.Find(str);
 
 	if (this->master) {
-		if (casep != NULL) {
-			strgen_error("Cases in the base translation are not supported.");
+		if (casep != nullptr) {
+			StrgenError("Cases in the base translation are not supported.");
 			return;
 		}
 
-		if (ent != NULL) {
-			strgen_error("String name '%s' is used multiple times", str);
+		if (ent != nullptr) {
+			StrgenError("String name '{}' is used multiple times", str);
 			return;
 		}
 
-		if (this->data.strings[this->data.next_string_id] != NULL) {
-			strgen_error("String ID 0x%X for '%s' already in use by '%s'", this->data.next_string_id, str, this->data.strings[this->data.next_string_id]->name);
+		if (this->data.strings[this->data.next_string_id] != nullptr) {
+			StrgenError("String ID 0x{:X} for '{}' already in use by '{}'", this->data.next_string_id, str, this->data.strings[this->data.next_string_id]->name);
 			return;
 		}
 
 		/* Allocate a new LangString */
-		this->data.Add(str, new LangString(str, s, this->data.next_string_id++, _cur_line));
+		this->data.Add(std::make_unique<LangString>(str, s, this->data.next_string_id++, _cur_line));
 	} else {
-		if (ent == NULL) {
-			strgen_warning("String name '%s' does not exist in master file", str);
+		if (ent == nullptr) {
+			StrgenWarning("String name '{}' does not exist in master file", str);
 			return;
 		}
 
-		if (ent->translated && casep == NULL) {
-			strgen_error("String name '%s' is used multiple times", str);
+		if (!ent->translated.empty() && casep == nullptr) {
+			StrgenError("String name '{}' is used multiple times", str);
 			return;
 		}
 
 		/* make sure that the commands match */
-		if (!CheckCommandsMatch(s, ent->english, str)) return;
+		if (!CheckCommandsMatch(s, ent->english.c_str(), str)) return;
 
-		if (casep != NULL) {
-			ent->translated_case = new Case(ResolveCaseName(casep, strlen(casep)), s, ent->translated_case);
+		if (casep != nullptr) {
+			ent->translated_cases.emplace_back(ResolveCaseName(casep, strlen(casep)), s);
 		} else {
-			ent->translated = stredup(s);
+			ent->translated = s;
 			/* If the string was translated, use the line from the
 			 * translated language so errors in the translated file
 			 * are properly referenced to. */
@@ -801,27 +731,28 @@ void StringReader::HandlePragma(char *str)
 	if (!memcmp(str, "plural ", 7)) {
 		_lang.plural_form = atoi(str + 7);
 		if (_lang.plural_form >= lengthof(_plural_forms)) {
-			strgen_fatal("Invalid pluralform %d", _lang.plural_form);
+			StrgenFatal("Invalid pluralform {}", _lang.plural_form);
 		}
 	} else {
-		strgen_fatal("unknown pragma '%s'", str);
+		StrgenFatal("unknown pragma '{}'", str);
 	}
 }
 
-static void rstrip(char *buf)
+static void StripTrailingWhitespace(std::string &str)
 {
-	size_t i = strlen(buf);
-	while (i > 0 && (buf[i - 1] == '\r' || buf[i - 1] == '\n' || buf[i - 1] == ' ')) i--;
-	buf[i] = '\0';
+	str.erase(str.find_last_not_of("\r\n ") + 1);
 }
 
 void StringReader::ParseFile()
 {
-	char buf[2048];
 	_warnings = _errors = 0;
 
-	_translation = this->master || this->translation;
-	_file = this->file;
+	_translation = this->translation;
+	_file = this->file.c_str();
+
+	/* Abusing _show_todo to replace "warning" with "info" for translations. */
+	_show_todo &= 3;
+	if (!this->translation) _show_todo |= 4;
 
 	/* For each new file we parse, reset the genders, and language codes. */
 	MemSetT(&_lang, 0);
@@ -830,11 +761,18 @@ void StringReader::ParseFile()
 	strecpy(_lang.digit_decimal_separator, ".", lastof(_lang.digit_decimal_separator));
 
 	_cur_line = 1;
-	while (this->ReadLine(buf, lastof(buf)) != NULL) {
-		rstrip(buf);
-		this->HandleString(buf);
+	while (this->data.next_string_id < this->data.max_strings) {
+		std::optional<std::string> line = this->ReadLine();
+		if (!line.has_value()) return;
+
+		StripTrailingWhitespace(line.value());
+		this->HandleString(line.value().data());
 		_cur_line++;
 	}
+
+	if (this->data.next_string_id == this->data.max_strings) {
+		StrgenError("Too many strings, maximum allowed is {}", this->data.max_strings);
+	}
 }
 
 /**
@@ -845,7 +783,7 @@ void HeaderWriter::WriteHeader(const StringData &data)
 {
 	int last = 0;
 	for (size_t i = 0; i < data.max_strings; i++) {
-		if (data.strings[i] != NULL) {
+		if (data.strings[i] != nullptr) {
 			this->WriteStringID(data.strings[i]->name, (int)i);
 			last = (int)i;
 		}
@@ -858,22 +796,22 @@ static int TranslateArgumentIdx(int argidx, int offset)
 {
 	int sum;
 
-	if (argidx < 0 || (uint)argidx >= lengthof(_cur_pcs.cmd)) {
-		strgen_fatal("invalid argidx %d", argidx);
+	if (argidx < 0 || (uint)argidx >= _cur_pcs.consuming_commands.max_size()) {
+		StrgenFatal("invalid argidx {}", argidx);
 	}
-	const CmdStruct *cs = _cur_pcs.cmd[argidx];
-	if (cs != NULL && cs->consumes <= offset) {
-		strgen_fatal("invalid argidx offset %d:%d", argidx, offset);
+	const CmdStruct *cs = _cur_pcs.consuming_commands[argidx];
+	if (cs != nullptr && cs->consumes <= offset) {
+		StrgenFatal("invalid argidx offset {}:{}", argidx, offset);
 	}
 
-	if (_cur_pcs.cmd[argidx] == NULL) {
-		strgen_fatal("no command for this argidx %d", argidx);
+	if (_cur_pcs.consuming_commands[argidx] == nullptr) {
+		StrgenFatal("no command for this argidx {}", argidx);
 	}
 
 	for (int i = sum = 0; i < argidx; i++) {
-		const CmdStruct *cs = _cur_pcs.cmd[i];
+		cs = _cur_pcs.consuming_commands[i];
 
-		sum += (cs != NULL) ? cs->consumes : 1;
+		sum += (cs != nullptr) ? cs->consumes : 1;
 	}
 
 	return sum + offset;
@@ -901,7 +839,7 @@ static void PutCommandString(Buffer *buffer, const char *str)
 		int argno;
 		int casei;
 		const CmdStruct *cs = ParseCommandString(&str, param, &argno, &casei);
-		if (cs == NULL) break;
+		if (cs == nullptr) break;
 
 		if (casei != -1) {
 			buffer->AppendUtf8(SCC_SET_CASE); // {SET_CASE}
@@ -917,9 +855,9 @@ static void PutCommandString(Buffer *buffer, const char *str)
 			}
 
 			/* Output the one from the master string... it's always accurate. */
-			cs = _cur_pcs.cmd[_cur_argidx++];
-			if (cs == NULL) {
-				strgen_fatal("%s: No argument exists at position %d", _cur_ident, _cur_argidx - 1);
+			cs = _cur_pcs.consuming_commands[_cur_argidx++];
+			if (cs == nullptr) {
+				StrgenFatal("{}: No argument exists at position {}", _cur_ident, _cur_argidx - 1);
 			}
 		}
 
@@ -936,7 +874,7 @@ void LanguageWriter::WriteLength(uint length)
 	char buffer[2];
 	int offs = 0;
 	if (length >= 0x4000) {
-		strgen_fatal("string too long");
+		StrgenFatal("string too long");
 	}
 
 	if (length >= 0xC0) {
@@ -952,16 +890,16 @@ void LanguageWriter::WriteLength(uint length)
  */
 void LanguageWriter::WriteLang(const StringData &data)
 {
-	uint *in_use = AllocaM(uint, data.tabs);
+	std::vector<uint> in_use;
 	for (size_t tab = 0; tab < data.tabs; tab++) {
 		uint n = data.CountInUse((uint)tab);
 
-		in_use[tab] = n;
+		in_use.push_back(n);
 		_lang.offsets[tab] = TO_LE16(n);
 
 		for (uint j = 0; j != in_use[tab]; j++) {
-			const LangString *ls = data.strings[(tab * TAB_SIZE) + j];
-			if (ls != NULL && ls->translated == NULL) _lang.missing++;
+			const LangString *ls = data.strings[(tab * TAB_SIZE) + j].get();
+			if (ls != nullptr && ls->translated.empty()) _lang.missing++;
 		}
 	}
 
@@ -975,23 +913,22 @@ void LanguageWriter::WriteLang(const StringData &data)
 
 	for (size_t tab = 0; tab < data.tabs; tab++) {
 		for (uint j = 0; j != in_use[tab]; j++) {
-			const LangString *ls = data.strings[(tab * TAB_SIZE) + j];
-			const Case *casep;
-			const char *cmdp;
+			const LangString *ls = data.strings[(tab * TAB_SIZE) + j].get();
+			const std::string *cmdp;
 
 			/* For undefined strings, just set that it's an empty string */
-			if (ls == NULL) {
+			if (ls == nullptr) {
 				this->WriteLength(0);
 				continue;
 			}
 
-			_cur_ident = ls->name;
+			_cur_ident = ls->name.c_str();
 			_cur_line = ls->line;
 
 			/* Produce a message if a string doesn't have a translation. */
-			if (_show_todo > 0 && ls->translated == NULL) {
+			if (_show_todo > 0 && ls->translated.empty()) {
 				if ((_show_todo & 2) != 0) {
-					strgen_warning("'%s' is untranslated", ls->name);
+					StrgenWarning("'{}' is untranslated", ls->name);
 				}
 				if ((_show_todo & 1) != 0) {
 					const char *s = "<TODO> ";
@@ -1000,53 +937,46 @@ void LanguageWriter::WriteLang(const StringData &data)
 			}
 
 			/* Extract the strings and stuff from the english command string */
-			ExtractCommandString(&_cur_pcs, ls->english, false);
+			_cur_pcs = ExtractCommandString(ls->english.c_str(), false);
 
-			if (ls->translated_case != NULL || ls->translated != NULL) {
-				casep = ls->translated_case;
-				cmdp = ls->translated;
+			if (!ls->translated_cases.empty() || !ls->translated.empty()) {
+				cmdp = &ls->translated;
 			} else {
-				casep = NULL;
-				cmdp = ls->english;
+				cmdp = &ls->english;
 			}
 
-			_translated = cmdp != ls->english;
-
-			if (casep != NULL) {
-				const Case *c;
-				uint num;
+			_translated = cmdp != &ls->english;
 
+			if (!ls->translated_cases.empty()) {
 				/* Need to output a case-switch.
 				 * It has this format
 				 * <0x9E> <NUM CASES> <CASE1> <LEN1> <STRING1> <CASE2> <LEN2> <STRING2> <CASE3> <LEN3> <STRING3> <STRINGDEFAULT>
 				 * Each LEN is printed using 2 bytes in big endian order. */
 				buffer.AppendUtf8(SCC_SWITCH_CASE);
-				/* Count the number of cases */
-				for (num = 0, c = casep; c; c = c->next) num++;
-				buffer.AppendByte(num);
+				buffer.AppendByte((byte)ls->translated_cases.size());
 
 				/* Write each case */
-				for (c = casep; c != NULL; c = c->next) {
-					buffer.AppendByte(c->caseidx);
+				for (const Case &c : ls->translated_cases) {
+					buffer.AppendByte(c.caseidx);
 					/* Make some space for the 16-bit length */
-					uint pos = buffer.Length();
+					uint pos = (uint)buffer.size();
 					buffer.AppendByte(0);
 					buffer.AppendByte(0);
 					/* Write string */
-					PutCommandString(&buffer, c->string);
+					PutCommandString(&buffer, c.string.c_str());
 					buffer.AppendByte(0); // terminate with a zero
 					/* Fill in the length */
-					uint size = buffer.Length() - (pos + 2);
+					uint size = (uint)buffer.size() - (pos + 2);
 					buffer[pos + 0] = GB(size, 8, 8);
 					buffer[pos + 1] = GB(size, 0, 8);
 				}
 			}
 
-			if (cmdp != NULL) PutCommandString(&buffer, cmdp);
+			if (!cmdp->empty()) PutCommandString(&buffer, cmdp->c_str());
 
-			this->WriteLength(buffer.Length());
-			this->Write(buffer.Begin(), buffer.Length());
-			buffer.Clear();
+			this->WriteLength((uint)buffer.size());
+			this->Write(buffer.data(), buffer.size());
+			buffer.clear();
 		}
 	}
 }
